@@ -308,18 +308,15 @@ _dbus_babysitter_unref (DBusBabysitter *sitter)
           if (ret == 0)
             kill (sitter->sitter_pid, SIGKILL);
 
+        again:
           if (ret == 0)
-            {
-              do
-                {
-                  ret = waitpid (sitter->sitter_pid, &status, 0);
-                }
-              while (_DBUS_UNLIKELY (ret < 0 && errno == EINTR));
-            }
+            ret = waitpid (sitter->sitter_pid, &status, 0);
 
           if (ret < 0)
             {
-              if (errno == ECHILD)
+              if (errno == EINTR)
+                goto again;
+              else if (errno == ECHILD)
                 _dbus_warn ("Babysitter process not available to be reaped; should not happen\n");
               else
                 _dbus_warn ("Unexpected error %d in waitpid() for babysitter: %s\n",
@@ -1259,7 +1256,11 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
           _dbus_assert_not_reached ("Got to code after write_err_and_exit()");
 	}
       else if (grandchild_pid == 0)
-	{
+      {
+          /* Go back to ignoring SIGPIPE, since it's evil
+           */
+          signal (SIGPIPE, SIG_IGN);
+
 	  do_exec (child_err_report_pipe[WRITE_END],
 		   argv,
 		   env,
